﻿Imports System.Threading

Public Class Form1

    Dim serialport As New System.IO.Ports.SerialPort    ' Declares a handle for COM port
    Dim serBuf(20) As Byte                              ' A global accessable array for data to be buffered for the COMX port
    Dim states As Integer = 0

    Private Sub Form1_Load(sender As Object, e As EventArgs) Handles MyBase.Load
        ' Show all available COM ports.
        For Each sp As String In My.Computer.Ports.SerialPortNames
            ComboBox_comport().Items.Add(sp)            ' Adds COMX to listbox called "comportselect"
        Next
        ComboBox_comport.Text = "Select COM Port"       ' Adds the message when program is first loaded
    End Sub

    Private Sub ComboBox_comport_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ComboBox_comport.SelectedIndexChanged
        If (serialport.IsOpen) Then                                                                 ' If program has previously opened a COM port, start by closing it
            serialport.Close()
        End If
        Try
            serialport = My.Computer.Ports.OpenSerialPort(ComboBox_comport.Text, 19200, 0, 8, 2)    ' attempt to open a new COM port
        Catch                                                                                       ' Catch open failures
            MessageBox.Show("Failed to open COM port", "USB-I2C", MessageBoxButtons.OKCancel, MessageBoxIcon.Asterisk)
            Return
        End Try
        serialport.ReadTimeout = 1000                                                               ' We will only arrive here on successfull open
        Timer1.Enabled = True
    End Sub

    Private Sub transmit(num As Integer)
        If serialport.IsOpen Then
            Try
                serialport.Write(serBuf, 0, num)            ' writes specified amount of SerBuf out on COM port
            Catch
                serialport.Close()
                Timer1.Enabled = False
                MessageBox.Show("Failed to read from COM port", "ERROR!", MessageBoxButtons.OKCancel, MessageBoxIcon.Asterisk)
                Return
            End Try
        End If
    End Sub

    Private Sub receive(num As Integer)
        If serialport.IsOpen Then
            Dim x As Integer = 0
            For x = 0 To num - 1                            ' Call the read function for passed number of times
                Try                                         ' This way we receive the correct number of bytes while still using timeouts
                    serialport.Read(serBuf, x, 1)
                Catch ex As Exception
                    serialport.Close()
                    Timer1.Enabled = False
                    MessageBox.Show("Failed to read from COM port", "ERROR!", MessageBoxButtons.OKCancel, MessageBoxIcon.Asterisk)
                End Try
            Next x
        End If
    End Sub

    Private Sub Timer1_Tick(sender As Object, e As EventArgs) Handles Timer1.Tick
        serBuf(0) = &H55    ' Perform a ranging with the SRF08 in CM
        serBuf(1) = &HE0
        serBuf(2) = &H0
        serBuf(3) = &H1
        serBuf(4) = &H51
        transmit(5)
        receive(1)

        Thread.Sleep(75)    ' Pause to allow ranging to complete

        serBuf(0) = &H55    ' Read the data back
        serBuf(1) = &HE1
        serBuf(2) = &H0
        serBuf(3) = &H4
        transmit(4)
        receive(4)

        ' Display the data
        Label_v.Text = String.Format("SRF08 v: {0}", serBuf(0))
        Label_light.Text = String.Format("Light value: {0}", serBuf(1))
        Dim range As Integer
        range = serBuf(2)   ' Put the range together from the high and low bytes
        range <<= 8
        range += serBuf(3)
        Label_range.Text = String.Format("Range: {0}", range)
    End Sub
End Class
